//+------------------------------------------------------------------
//|
//| Derivative Oscillator as published by Constance Brown
//| "Technical Analysis for the trading professional"
//| Low lag oscillator which is useful at divergence analysis.
//+------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1  DeepSkyBlue
#property indicator_color2  DeepSkyBlue
#property indicator_color3  PaleVioletRed
#property indicator_color4  PaleVioletRed
#property indicator_color5  DimGray
#property indicator_width1  2
#property indicator_width4  2
#property indicator_width5  2

//
//
//
//
//

extern string TimeFrame          = "Current time frame";
extern int    RsiPeriod          = 14;
extern int    RsiPrice           = PRICE_CLOSE;
extern int    EmaSmoothing1      = 15;
extern int    EmaSmoothing2      =  3;
extern int    SmaSmoothing       =  9;
extern bool   alertsOn           = false;
extern bool   alertsOnCurrent    = false;
extern bool   alertsMessage      = true;
extern bool   alertsNotification = false;
extern bool   alertsSound        = false;
extern bool   alertsEmail        = false;
extern bool   Interpolate        = false;

//
//
//
//
//

double der[];
double derUu[];
double derUd[];
double derDd[];
double derDu[];
double trend[];
double slope[];

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------
//|
//+------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorDigits(5);
   IndicatorBuffers(7);
      SetIndexBuffer(0,derUu); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,derUd); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,derDu); SetIndexStyle(2,DRAW_HISTOGRAM);
      SetIndexBuffer(3,derDd); SetIndexStyle(3,DRAW_HISTOGRAM);
      SetIndexBuffer(4,der);
      SetIndexBuffer(5,trend);
      SetIndexBuffer(6,slope);

      //
      //
      //
      //
      //
            
      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
   IndicatorShortName(timeFrameToString(timeFrame)+" Derivative oscillator ("+RsiPeriod+","+EmaSmoothing1+","+EmaSmoothing2+","+SmaSmoothing+")");
   return(0);
}

//+------------------------------------------------------------------
//|
//+------------------------------------------------------------------
//
//
// 
//
// 
//
//

double work[][2];
#define _ema1 0
#define _ema2 1

//
//
//
//
//

int start() 
{
   int i,k,n,r,limit,counted_bars = IndicatorCounted();

      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            limit = MathMin(Bars-counted_bars,Bars-1);
            if (returnBars) { derUu[0] = limit; return(0); }
   	
   //
   //
   //
   //
   //
      	
   if (calculateValue || timeFrame==Period())
   {      	
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
      double alpha1 = 2.0/(1.0+EmaSmoothing1);
      double alpha2 = 2.0/(1.0+EmaSmoothing2);
      
      //
      //
      //
      //
      //
      
      for(i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         double rsi = iRSI(NULL,0,RsiPeriod,RsiPrice,i);
                      work[r][_ema1] = work[r-1][_ema1]+alpha1*(rsi           -work[r-1][_ema1]);
                      work[r][_ema2] = work[r-1][_ema2]+alpha2*(work[r][_ema1]-work[r-1][_ema2]);
         double sma = work[r][_ema2]; for (k=1; k<SmaSmoothing && (r-k)>=0; k++) sma += work[r-k][_ema2]; sma /= k;

         //
         //
         //
         //
         //
      
         der[i] = work[r][_ema2]-sma;
         derUu[i] = EMPTY_VALUE;
         derUd[i] = EMPTY_VALUE;
         derDu[i] = EMPTY_VALUE;
         derDd[i] = EMPTY_VALUE;
         trend[i] = trend[i+1];
         slope[i] = slope[i+1];
            if (der[i]>0)        trend[i] =  1;
            if (der[i]<0)        trend[i] = -1;
            if (der[i]>der[i+1]) slope[i] =  1;
            if (der[i]<der[i+1]) slope[i] = -1;
            if (trend[i] == 1)
               if (slope[i]==1)
                     derUu[i] = der[i];
               else  derUd[i] = der[i];
            if (trend[i] == -1)
               if (slope[i]==1)
                     derDu[i] = der[i];
               else  derDd[i] = der[i];
      }
      manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         der[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RsiPeriod,RsiPrice,EmaSmoothing1,EmaSmoothing2,SmaSmoothing,alertsOn,alertsOnCurrent,alertsMessage,alertsNotification,alertsSound,alertsEmail,4,y);
         trend[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RsiPeriod,RsiPrice,EmaSmoothing1,EmaSmoothing2,SmaSmoothing,alertsOn,alertsOnCurrent,alertsMessage,alertsNotification,alertsSound,alertsEmail,5,y);
         slope[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RsiPeriod,RsiPrice,EmaSmoothing1,EmaSmoothing2,SmaSmoothing,alertsOn,alertsOnCurrent,alertsMessage,alertsNotification,alertsSound,alertsEmail,6,y);     
         derUu[i] = EMPTY_VALUE;
         derUd[i] = EMPTY_VALUE;
         derDu[i] = EMPTY_VALUE;
         derDd[i] = EMPTY_VALUE;
          
         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(k = 1; k < n; k++)
               der[i+k] = der[i] + (der[i+n]-der[i])*k/n;
   }              
   for(i=limit; i>=0; i--)
   {
      if (trend[i] == 1)
               if (slope[i]==1)
                     derUu[i] = der[i];
               else  derUd[i] = der[i];
      if (trend[i] == -1)
               if (slope[i]==1)
                     derDu[i] = der[i];
               else  derDd[i] = der[i];
   }               
   return(0);
}



//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   StringToUpper(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      
      //
      //
      //
      //
      //
      
      static datetime prevTime1  = 0;
      static string   prevAlert1 = "";
         if (trend[whichBar] != trend[whichBar+1])
         {
            if (trend[whichBar] ==  1) doAlert(prevTime1,prevAlert1," Derivative oscillator crossed zero line up");
            if (trend[whichBar] == -1) doAlert(prevTime1,prevAlert1," Derivative oscillator crossed zero line down");
         }
   }
}

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       //
       //
       //
       //
       //

       message = timeFrameToString(Period())+" - "+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+doWhat;
          if (alertsMessage)      Alert(message);
          if (alertsNotification) SendNotification(message);
          if (alertsEmail)        SendMail(StringConcatenate(Symbol(),"Derivative oscillator "),message);
          if (alertsSound)        PlaySound("alert2.wav");
   }
}

